import QtQuick 2.2
import Ubuntu.Components 1.1
import Filters 1.0

Item {
    id: editor
    property url source
    property url transformPreviewSource
    property bool showOriginal: mouse.pressed

    function saveToFile(fileName) {
        fullImage.source = editor.source;
        // FIXME: fullImage.image is not guaranteed to be set at this point
        transformPreview.item.saveTransform.input = fullImage.image;
        transformPreview.item.saveTransform.saveToFile(fileName);
        transformPreview.item.saveTransform.input = undefined;
        fullImage.source = "";
    }

    Rectangle {
        anchors.fill: parent
        color: "black"
    }

    MouseArea {
        id: mouse
        anchors.fill: parent
    }

    // preview pipeline
    ImageLoader {
        id: downscaledImage
        source: editor.source
        sourceSize {
            width: parent.width
            height: parent.height
        }
    }

    ImageRenderer {
        id: originalImage
        anchors.fill: parent
        image: downscaledImage.image
        anchors.centerIn: parent
        visible: showOriginal
    }

    TextureRenderer {
        id: transformedImage
        anchors.fill: parent
        textureProvider: transformPreview.item.previewTransform
        anchors.centerIn: parent
        visible: !showOriginal
    }

    Loader {
        id: transformPreview
        anchors.fill: parent
        source: editor.transformPreviewSource

        Binding {
            target: transformPreview.item
            property: "imageLoader"
            value: downscaledImage
        }
    }

    // save pipeline
    ImageLoader {
        id: fullImage
    }
}
