import QtQuick 2.2
import Ubuntu.Components 1.1

TransformPreview {
    id: transformPreview

    property real minimumValue: 0.0
    property real maximumValue: 1.0
    property real defaultValue: 1.0
    property real valueWidth: sliderWidth.value
    property real valueHeight: sliderHeight.value
    property real valueX: sliderX.value
    property real valueY: sliderY.value

    function formatValue(v) {
        return v.toFixed(0);
    }

    Label {
        id: labelWidth
        text: "Width"
        anchors {
            left: parent.left
            leftMargin: units.gu(2)
            verticalCenter: sliderWidth.verticalCenter
        }
    }

    Slider {
        id: sliderWidth
        anchors {
            left: labelWidth.right
            leftMargin: units.gu(2)
            right: parent.right
            rightMargin: units.gu(2)
            bottom: sliderHeight.top
        }
        minimumValue: transformPreview.minimumValue
        maximumValue: transformPreview.maximumValue
        value: transformPreview.defaultValue
        live: true
        function formatValue(v) {
            return transformPreview.formatValue(v);
        }
    }

    Label {
        id: labelHeight
        text: "Height"
        anchors {
            left: parent.left
            leftMargin: units.gu(2)
            verticalCenter: sliderHeight.verticalCenter
        }
    }

    Slider {
        id: sliderHeight
        anchors {
            left: labelHeight.right
            leftMargin: units.gu(2)
            right: parent.right
            rightMargin: units.gu(2)
            bottom: sliderX.top
        }
        minimumValue: transformPreview.minimumValue
        maximumValue: transformPreview.maximumValue
        value: transformPreview.defaultValue
        live: true
        function formatValue(v) {
            return transformPreview.formatValue(v);
        }
    }

    Label {
        id: labelX
        text: "Start X"
        anchors {
            left: parent.left
            leftMargin: units.gu(2)
            verticalCenter: sliderX.verticalCenter
        }
    }

    Slider {
        id: sliderX
        anchors {
            left: labelX.right
            leftMargin: units.gu(2)
            right: parent.right
            rightMargin: units.gu(2)
            bottom: sliderY.top
        }
        minimumValue: transformPreview.minimumValue
        maximumValue: transformPreview.maximumValue
        value: 1 - transformPreview.defaultValue
        live: true
        function formatValue(v) {
            return transformPreview.formatValue(v);
        }
    }

    Label {
        id: labelY
        text: "Start Y"
        anchors {
            left: parent.left
            leftMargin: units.gu(2)
            verticalCenter: sliderY.verticalCenter
        }
    }

    Slider {
        id: sliderY
        anchors {
            left: labelY.right
            leftMargin: units.gu(2)
            right: parent.right
            rightMargin: units.gu(2)
            bottom: parent.bottom
        }
        minimumValue: transformPreview.minimumValue
        maximumValue: transformPreview.maximumValue
        value: 1 - transformPreview.defaultValue
        live: true
        function formatValue(v) {
            return transformPreview.formatValue(v);
        }
    }
}
