import QtQuick 2.2
import Ubuntu.Components 1.1
import Filters 1.0

MainView {
    width: 1280
    height: 720
    useDeprecatedToolbar: false
    automaticOrientation: true

    Editor {
        id: editor
        anchors.fill: parent
        source: "parrot_head.jpg"
        transformPreviewSource: effects.get(effectsSelector.selectedIndex).qml
    }


    ListModel {
        id: effects
        ListElement { name: "Brightness"; qml: "TransformPreviewBrightnessHalide.qml"}
        ListElement { name: "Saturation"; qml: "TransformPreviewSaturationHalide.qml"}
        ListElement { name: "Rotation"; qml: "TransformPreviewRotationHalide.qml"}
        ListElement { name: "Blur"; qml: "TransformPreviewBoxBlurHalide.qml"}
        ListElement { name: "Crop"; qml: "TransformPreviewCropHalide.qml"}
    }



    Component.onCompleted: {
        Theme.palette.selected.backgroundText = "white";
    }

    Rectangle {
        id: overlay
        anchors {
            top: parent.top
            bottom: parent.bottom
            left: parent.left
        }
        width: units.gu(22)
        color: Qt.rgba(0.0, 0.0, 0.0, 0.2)

        Column {
            spacing: units.gu(2)
            anchors {
                fill: parent
                margins: units.gu(2)
            }

            Button {
                id: saveButton
                anchors {
                    left: parent.left
                    right: parent.right
                }

                property string outputFile: "output.png"
                text: "Save to %1".arg(outputFile)
                onClicked: editor.saveToFile(outputFile)
                strokeColor: "grey"
            }

            OptionSelector {
                id: effectsSelector
                anchors {
                    left: parent.left
                    right: parent.right
                }

                model: effects
                expanded: true
                delegate: OptionSelectorDelegate { text: name }
            }
        }
    }
}
