#include <Halide.h>
#include <QRect>
#include <QDebug>
#include "halide_generators_common.h"
#include "halide_common.h"
#include <math.h>

namespace {

QRect getOutputGeometry(Halide::Internal::GeneratorBase* generator);

class Crop : public Halide::Generator<Crop> {
public:
    ImageParam input{ Halide::type_of<uint8_t>(), 3, "input" };

    Param<float> startX{ "startX", 0.0f };
    Param<float> startY{ "startY", 0.0f };
    Param<float> cropWidth{ "cropWidth", 1.0f };
    Param<float> cropHeight{ "cropHeight", 1.0f };

    Func build() override {
        Halide::Var x, y, c;
        Halide::Func function;
        function(x, y, c) = input(x, y, c);

        schedule_cpu_and_glsl(input, function, x, y, c);

        // FIXME: can this be put somewhere else?
        set_packed(input);
        set_packed(function.output_buffer());

        GeneratorHelperRegistry::outputGeometryFunctions[this] = &getOutputGeometry;

        return function;
    }
};

Halide::RegisterGenerator<Crop> register_crop{"crop"};

QRect getOutputGeometry(Halide::Internal::GeneratorBase* generator) {
    QRect out;
    Crop* crop = static_cast<Crop*>(generator);

    int width = crop->input.get().extent(0);
    int height = crop->input.get().extent(1);

    out.setX(static_cast<int>(crop->startX.get() * width));
    out.setY(static_cast<int>(crop->startY.get() * height));
    out.setWidth(std::max(static_cast<int>(crop->cropWidth.get() * width) - out.x(), 1));
    out.setHeight(std::max(static_cast<int>(crop->cropHeight.get() * height) - out.y(), 1));

    qDebug() << out;
    return out;
}

}
