#ifndef HALIDE_COMMON_H
#define HALIDE_COMMON_H

#include <QtCore/QRect>
#include <QtCore/QHash>
#include <QtGui/QImage>
#include <Halide.h>

namespace {

buffer_t* packedBuffer(uint8_t* pixels, int width, int height, int channels)
{
    buffer_t* buffer = new buffer_t;
    memset(buffer, 0, sizeof(buffer_t));
    buffer->host = pixels;
    buffer->elem_size = 1;
    buffer->extent[0] = width;
    buffer->extent[1] = height;
    buffer->extent[2] = channels;
    buffer->stride[0] = buffer->extent[2];
    buffer->stride[1] = buffer->extent[0] * buffer->stride[0];
    buffer->stride[2] = 1;
}

buffer_t* halideBufferFromQImage(QImage& image)
{
    image = image.convertToFormat(QImage::Format_RGB32);

    if (QSysInfo::ByteOrder == QSysInfo::BigEndian) {
        // FIXME
    }

    return packedBuffer(image.bits(), image.width(), image.height(), 4);
}

}

typedef QRect (*getOutputGeometry)(Halide::Internal::GeneratorBase*);

class GeneratorHelperRegistry
{
public:
    static QHash<Halide::Internal::GeneratorBase*, getOutputGeometry> outputGeometryFunctions;
};

#endif // HALIDE_COMMON_H


