#include "halide_effect.h"
#include "halide_common.h"
#include <QtCore/QDebug>
#include <QtCore/QMetaObject>
#include <QtCore/QMetaProperty>
#include <QtCore/QElapsedTimer>
#include <QtCore/QThread>
#include <QtCore/QStandardPaths>

HalideEffect::HalideEffect(QQuickItem *parent) :
    QQuickItem(parent),
    m_textureId(0),
    m_textureProvider(NULL),
    m_textureNeedsUpdate(true),
    m_transform(this, false, false)
{
    m_transform.setPropertiesSource(this);
    setFlag(QQuickItem::ItemHasContents);
    connect(&m_transform, SIGNAL(updateCompleted(uint,QSize,QImage)),
            this, SLOT(onUpdateCompleted(uint,QSize,QImage)));
}

HalideEffect::~HalideEffect()
{
}

void HalideEffect::scheduleUpdate()
{
    m_transform.scheduleUpdate();
}

QString HalideEffect::generatorName() const
{
    return m_generatorName;
}

void HalideEffect::setGeneratorName(QString generatorName)
{
    if (generatorName != m_generatorName) {
        m_generatorName = generatorName;
        m_transform.setGeneratorName(m_generatorName);
        Q_EMIT generatorNameChanged();
    }
}

void HalideEffect::onUpdateCompleted(uint textureId, QSize textureSize, QImage result)
{
    m_textureId = textureId;
    m_textureSize = textureSize;
    m_result = result;
    setSize(m_textureSize);
    m_textureNeedsUpdate = true;
    QQuickItem::update();
}

bool HalideEffect::isTextureProvider() const
{
    return true;
}

QSGTextureProvider* HalideEffect::textureProvider() const
{
    if (m_textureProvider == NULL) {
        HalideEffect* nonconst_this = const_cast<HalideEffect*>(this);
        nonconst_this->m_textureProvider = new HalideEffectTextureProvider;
        nonconst_this->updateTexture();
    }
    return m_textureProvider;
}

QSGNode* HalideEffect::updatePaintNode(QSGNode* oldNode, UpdatePaintNodeData* updatePaintNodeData)
{
    Q_UNUSED(oldNode)
    Q_UNUSED(updatePaintNodeData)
    updateTexture();
    return NULL;
}

void HalideEffect::updateTexture()
{
    if (m_textureNeedsUpdate && m_textureProvider != NULL) {
        QSGTexture* texture = NULL;
        if (m_textureId > 0) {
            texture = window()->createTextureFromId(m_textureId, m_textureSize);//, QQuickWindow::TextureOwnsGLTexture);
        } else {
            texture = window()->createTextureFromImage(m_result);
        }
        m_textureProvider->updateTexture(texture);
        m_textureNeedsUpdate = false;
    }
}
