#ifndef HALIDEEFFECT_H
#define HALIDEEFFECT_H

#include "halide_transform.h"
#include <QtCore/QObject>
#include <QtQuick/QSGTextureProvider>
#include <QtQuick/QQuickItem>

class HalideEffectTextureProvider : public QSGTextureProvider
{
    Q_OBJECT

public:
    explicit HalideEffectTextureProvider()
        : m_texture(0)
    {
    }

    void updateTexture(QSGTexture *texture) {
        if (m_texture != texture) {
            m_texture = texture;
            Q_EMIT textureChanged();
        }
    }

    QSGTexture *texture() const {
        if (m_texture) {
            m_texture->setHorizontalWrapMode(QSGTexture::ClampToEdge);
            m_texture->setVerticalWrapMode(QSGTexture::ClampToEdge);
        }

        return m_texture;
    }

private:
    QSGTexture *m_texture;
};


class HalideEffect : public QQuickItem
{
    Q_OBJECT

    Q_PROPERTY(QString generatorName READ generatorName WRITE setGeneratorName NOTIFY generatorNameChanged)

public:
    explicit HalideEffect(QQuickItem *parent = 0);
    virtual ~HalideEffect();

    Q_INVOKABLE void scheduleUpdate();

    // inherited from QQuickItem
    bool isTextureProvider() const;
    QSGTextureProvider* textureProvider() const;
    QSGNode* updatePaintNode(QSGNode* oldNode, UpdatePaintNodeData* updatePaintNodeData);

    QString generatorName() const;
    void setGeneratorName(QString generatorName);

Q_SIGNALS:
    void generatorNameChanged();

protected Q_SLOTS:
    void onUpdateCompleted(uint textureId, QSize textureSize, QImage result);
    void updateTexture();

private:
    QString m_generatorName;
    uint m_textureId;
    QSize m_textureSize;
    QImage m_result;
    HalideEffectTextureProvider* m_textureProvider;
    bool m_textureNeedsUpdate;
    HalideTransform m_transform;
};

#endif // HALIDEEFFECT_H

