#ifndef HALIDETRANSFORM_H
#define HALIDETRANSFORM_H

#include <QtCore/QObject>
#include <QtCore/QSharedPointer>
#include <QtCore/QSignalMapper>
#include <QtGui/QImage>
#include <QtGui/QOpenGLContext>
#include <QtGui/QOffscreenSurface>
#include <QtGui/QOpenGLFramebufferObject>
#include <QtQuick/QQuickWindow>
#include <Halide.h>

class HalideTransform : public QObject
{
    Q_OBJECT

    Q_PROPERTY(QString generatorName READ generatorName WRITE setGeneratorName NOTIFY generatorNameChanged)

public:
    explicit HalideTransform(QObject *parent = 0,
                             bool resultRequired = true, // FIXME: getter/setter instead
                             bool customContext = false);
    ~HalideTransform();

    Q_INVOKABLE bool saveToFile(QString fileName);
    Q_INVOKABLE void scheduleUpdate();

    QString generatorName() const;
    void setGeneratorName(QString generatorName);
    QObject* propertiesSource() const;
    void setPropertiesSource(QObject* propertiesSource);

Q_SIGNALS:
    void generatorNameChanged();
    void propertiesSourceChanged();
    void updateCompleted(uint textureId, QSize textureSize, QImage result);

protected Q_SLOTS:
    void onPropertyChanged(QString propertyName);
    void onWindowChanged(QQuickWindow* window);
    void onWindowContextCreated(QOpenGLContext* context);

protected:
    void findQtQuickGLContext();
    void createOpenGLContext(QOpenGLContext* context);
    void destroyOpenGLContext();
    void updateFunctionFromGenerator();
    void matchPropertiesToArguments();
    void connectToPropertyChange(QString propertyName);
    bool setArgumentFromProperty(QString propertyName, QVariant value);
    bool apply();
    void doUpdate();

private:
    std::unique_ptr<Halide::Internal::GeneratorBase> m_generator;
    Halide::Func m_function;
    QString m_generatorName;
    // images is necessary so that the buffers used as parameters to m_function
    // are still alive when we call realize()
    QHash<QString, QImage> m_images;
    QHash<QString, bool> m_propertiesValid;
    bool m_allPropertiesValid;
    bool m_openGL;
    bool m_customContext;
    QOpenGLContext m_context;
    QOffscreenSurface m_surface;
    Halide::Buffer m_outputBuffer;
    QOpenGLFramebufferObject* m_renderFbo;
    uint m_textureId;
    QSize m_textureSize;
    QImage m_result;
    bool m_resultRequired;
    QObject* m_propertiesSource;
    QList<QSharedPointer<QSignalMapper> > m_signalMappers;
};

#endif // HALIDETRANSFORM_H

