#include "image_loader.h"

ImageLoader::ImageLoader(QObject *parent) :
    QObject(parent),
    m_source(""),
    m_completed(false)
{
}

ImageLoader::~ImageLoader()
{
}

void ImageLoader::classBegin()
{
}

void ImageLoader::componentComplete()
{
    // FIXME: this is called before m_sourceSize has its final value leading
    // to many pictures reload
    m_completed = true;
    reloadSource();
}

QUrl ImageLoader::source() const
{
    return m_source;
}

QImage ImageLoader::image() const
{
    return m_image;
}

QSize ImageLoader::sourceSize() const
{
    return m_sourceSize;
}

QSize ImageLoader::actualSize() const
{
    return m_actualSize;
}

void ImageLoader::setSource(QUrl source)
{
    if (source != m_source) {
        m_source = source;
        reloadSource();
        Q_EMIT sourceChanged();
    }
}

void ImageLoader::setSourceSize(QSize sourceSize)
{
    if (sourceSize != m_sourceSize) {
        m_sourceSize = sourceSize;
        reloadSource();
        Q_EMIT sourceSizeChanged();
    }
}

void ImageLoader::reloadSource()
{
    if (!m_completed) {
        return;
    }

    if (m_source.isValid()) {
        m_reader.setFileName(m_source.path());
        QSize imageSize = m_reader.size();
        if (!m_sourceSize.isNull() &&
            (imageSize.width() > m_sourceSize.width() ||
             imageSize.height() > m_sourceSize.height())) {
            QSize validRequestedSize = m_sourceSize;
            if (validRequestedSize.width() == 0) {
                validRequestedSize.setWidth(imageSize.width());
            }
            if (validRequestedSize.height() == 0) {
                validRequestedSize.setHeight(imageSize.height());
            }
            imageSize.scale(validRequestedSize, Qt::KeepAspectRatio);
            m_reader.setScaledSize(imageSize);
        }
        m_actualSize = m_reader.size();
        m_image = m_reader.read();
        Q_EMIT actualSizeChanged();
        Q_EMIT imageChanged();
    } else {
        if (!m_image.isNull()) {
            m_image = QImage();
            m_actualSize = QSize(0, 0);
            Q_EMIT actualSizeChanged();
            Q_EMIT imageChanged();
        }
    }
}
