#ifndef IMAGELOADER_H
#define IMAGELOADER_H

#include <QObject>
#include <QImage>
#include <QImageReader>
#include <QUrl>
#include <QSize>
#include <QtQml/QQmlParserStatus>

class ImageLoader : public QObject, public QQmlParserStatus
{
    Q_OBJECT
    Q_INTERFACES(QQmlParserStatus)

    Q_PROPERTY(QUrl source READ source WRITE setSource NOTIFY sourceChanged)
    Q_PROPERTY(QImage image READ image NOTIFY imageChanged)
    Q_PROPERTY(QSize sourceSize READ sourceSize WRITE setSourceSize NOTIFY sourceSizeChanged)
    Q_PROPERTY(QSize actualSize READ actualSize NOTIFY actualSizeChanged)

public:
    explicit ImageLoader(QObject *parent = 0);
    ~ImageLoader();

    // inherited from QQmlParserStatus
    virtual void classBegin();
    virtual void componentComplete();

    QUrl source() const;
    QImage image() const;
    QSize sourceSize() const;
    QSize actualSize() const;

    void setSource(QUrl source);
    void setSourceSize(QSize sourceSize);

Q_SIGNALS:
    void sourceChanged();
    void imageChanged();
    void sourceSizeChanged();
    void actualSizeChanged();

protected:
    void reloadSource();

private:
    bool m_completed;
    QUrl m_source;
    QImage m_image;
    QSize m_sourceSize;
    QSize m_actualSize;
    QImageReader m_reader;
};

#endif // IMAGELOADER_H

