#ifndef IMAGETRANSFORM_H
#define IMAGETRANSFORM_H

#include <QObject>
#include <QImage>
#include <QFutureWatcher>

class ImageTransform : public QObject
{
    Q_OBJECT

    Q_PROPERTY(QImage sourceImage READ sourceImage WRITE setSourceImage NOTIFY sourceImageChanged)
    Q_PROPERTY(QImage result READ result NOTIFY resultChanged)
    Q_PROPERTY(bool hasResult READ hasResult NOTIFY hasResultChanged)

public:
    explicit ImageTransform(QObject *parent = 0);
    virtual ~ImageTransform();

    QImage sourceImage() const;
    QImage result() const;
    bool hasResult() const;
    void setSourceImage(QImage sourceImage);

    Q_INVOKABLE bool saveToFile(QString fileName);
    Q_INVOKABLE virtual void scheduleUpdate();

Q_SIGNALS:
    void sourceImageChanged();
    void resultChanged();
    void hasResultChanged();

protected:
    // prepare is guaranteed to be run just in the main thread
    Q_INVOKABLE virtual void prepare(void);
    // transform is run in a thread that is not the main thread
    virtual QImage transform(QImage input);
    void doUpdate();
    void setHasResult(bool hasResult);

protected Q_SLOTS:
    void updateFinished();

private:
    QImage m_sourceImage;
    QImage m_result;
    bool m_updatePending;
    bool m_hasResult;
    QFutureWatcher<void> m_updateWatcher;
};

#endif // IMAGETRANSFORM_H

