#include <Halide.h>
#include "halide_generators_common.h"

namespace {

class Rotation : public Halide::Generator<Rotation> {
public:
    ImageParam input{ Halide::type_of<uint8_t>(), 3, "input" };
    Param<float> angle{ "angle", 1.0 };

    Func build() override {
        Halide::Expr alpha = -1.0f * angle * M_PI / 180.0; // to radians

        Halide::Var x, y, c;

        Halide::Expr centerX = input.width() / 2;
        Halide::Expr centerY = input.height() / 2;

        Halide::Expr offsetX = input.width() / 2;
        Halide::Expr offsetY = input.height() / 2;

        Halide::Expr newX = Halide::cast<int>((x - offsetX)*Halide::cos(alpha) - Halide::sin(alpha)*(y - offsetY) + centerX);
        Halide::Expr newY = Halide::cast<int>((x - offsetX)*Halide::sin(alpha) + Halide::cos(alpha)*(y - offsetY) + centerY);

        Halide::Expr realNewX = clamp(newX, 0, input.width() - 1);
        Halide::Expr realNewY = clamp(newY, 0, input.height() - 1);

        Halide::Expr inBounds = (newX > 0 && newX < input.width() &&
                                 newY > 0 && newY < input.height());
        Halide::Expr imagePixel = input(realNewX, realNewY, c);

        Halide::Func function;
        function(x, y, c) = Halide::cast<uint8_t>(select(inBounds, imagePixel, 0));

        schedule_cpu_and_glsl(input, function, x, y, c);

        // FIXME: can this be put somewhere else?
        set_packed(input);
        set_packed(function.output_buffer());


        return function;
    }

};

Halide::RegisterGenerator<Rotation> register_rotation{"rotation"};

}
