#include <Halide.h>
#include "halide_generators_common.h"

namespace {

class Saturation : public Halide::Generator<Saturation> {
public:
    ImageParam input{ Halide::type_of<uint8_t>(), 3, "input" };
    // between 0.0f (greyscale) and 2.0f (1.0f for no change)
    Param<float> saturation{ "saturation", 1.0 };

    Func build() override {
        Halide::Func function;
        Halide::Var x, y, c;
        Halide::Expr grey = 0.3f*input(x,y,0) + 0.59f*input(x,y,1) + 0.11f*input(x,y,2);
        Halide::Expr saturate_float = Halide::lerp(grey, Halide::cast<float>(input(x, y, c)), saturation );
        Halide::Expr saturate = Halide::cast<uint8_t>(Halide::clamp(saturate_float, 0.0f, 255.0f));
        function(x, y, c) = select(c == 3, cast<uint8_t>(255), saturate);

        schedule_cpu_and_glsl(input, function, x, y, c);

        // FIXME: can this be put somewhere else?
        set_packed(input);
        set_packed(function.output_buffer());

        return function;
    }

};

Halide::RegisterGenerator<Saturation> register_saturation{"saturation"};

}
