#include "saturation_transform.h"

SaturationTransform::SaturationTransform(QObject *parent) :
    ImageTransform(parent)
{

}

SaturationTransform::~SaturationTransform()
{

}

float SaturationTransform::saturation() const
{
    return m_saturation;
}

void SaturationTransform::setSaturation(float saturation)
{
    if (saturation != m_saturation) {
        m_saturation = saturation;
        scheduleUpdate();
        Q_EMIT saturationChanged();
    }
}

QImage SaturationTransform::transform(QImage input)
{
    float saturation = m_saturation / 100.0f;
    QImage output = QImage(input.size(), input.format());
    unsigned int nPixels = input.width() * input.height();
    const QRgb* inputPixels = (const QRgb *)input.constBits();
    QRgb* outputPixels = (QRgb *)output.bits();

    for (unsigned int i=0; i<nPixels; i++) {
        QRgb in = inputPixels[i];
        int r = qRed(in);
        int g = qGreen(in);
        int b = qBlue(in);
        int gray = (r*11+g*16+b*5)/32;
        QRgb out = qRgb( qBound(0, r + (int)((r - gray) * saturation), 255),
                         qBound(0, g + (int)((g - gray) * saturation), 255),
                         qBound(0, b + (int)((b - gray) * saturation), 255));
        outputPixels[i] = out;
    }

    return output;
}
